/*
* lstrip.c
* compress Lua programs by removing comments and whitespaces, but preserving
* line breaks (for error messages). Uses the internals of Lua politely.
* 16 Oct 2002 11:36:47
*/

#include <ctype.h>
#include <stdio.h>

#include "lua.h"
#include "llex.h"
#include "lstring.h"
#include "lzio.h"

static int error(lua_State *L) 
{
 fprintf(stderr,"%s\n",lua_tostring(L,-1));
 return 0;
}

static const char *reader (lua_State *L, void *data, size_t *size)
{
 static char buff[512];
 FILE *f = (FILE*) data;
 if (feof(f)) return NULL;
 *size = fread(buff, 1, sizeof(buff), f);
 return (*size > 0) ? buff : NULL;
}

static void quote(const char* s)
{
 putchar('"');
 for (; *s; s++)
 {
  switch (*s)
  {
   case '"': printf("\\\""); break;
   case '\a': printf("\\a"); break;
   case '\b': printf("\\b"); break;
   case '\f': printf("\\f"); break;
   case '\n': printf("\\n"); break;
   case '\r': printf("\\r"); break;
   case '\t': printf("\\t"); break;
   case '\v': printf("\\v"); break;
   default: putchar(*s); break;
  }
 }
 putchar('"');
}

#define NEEDSPACE(t)	(((t)>=FIRST_RESERVED && (t)<=TK_NAME) || (t)==TK_NUMBER)

static void strip(FILE* f, const char* filename)
{
 ZIO z;
 int ln=1;
 int lt=0;
 Mbuffer buff;
 LexState lexstate;
 lua_State* L=lua_open();
 lua_atpanic(L,error);
 luaZ_init(&z,reader,f,filename);
 luaZ_initbuffer(L,&buff);
 lexstate.buff=&buff;
 luaX_setinput(L,&lexstate,&z,luaS_new(L,filename));
 while (1)
 {
  int t=luaX_lex(&lexstate,&lexstate.t.seminfo);
  if (lexstate.linenumber!=ln)
  {
   while (lexstate.linenumber!=ln++) printf("\n");
   ln=lexstate.linenumber;
   lt=0;
  }
  if (NEEDSPACE(lt) && NEEDSPACE(t)) printf(" ");
  switch (t)
  {
   case TK_EOS:
    return;
   case TK_STRING:
    quote(getstr(lexstate.t.seminfo.ts));
    break;
   case TK_NAME:
    printf("%s",getstr(lexstate.t.seminfo.ts));
    break;
   case TK_NUMBER:
#if 0
    printf("%.16g",lexstate.t.seminfo.r);
#else
    printf("%s",lexstate.buff->buffer);
#endif
    break;
   case TK_CONCAT:
    if (lt==TK_NUMBER) printf(" ");
   default:
    printf("%s",luaX_token2str(&lexstate,t));
    break;
  }
  lt=t;
 }
}

int main(void)
{
 strip(stdin,"=stdin");
 return 0;
}
